@@ -8,6 +8,9 @@
  *
  * Authors:	Alexey Kuznetsov, <kuznet@ms2.inr.ac.ru>
  *
+ * Changes:	Arnaldo Carvalho de Melo <acme@conectiva.com.br>
+ *		20010404 - use setlocale
+ *
  */
 
 #include <stdio.h>
@@ -17,15 +20,15 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
-#include <linux/netdevice.h>
-#include <linux/if.h>
-#include <linux/if_arp.h>
-#include <linux/sockios.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <string.h>
 
+#include <net/if.h>
+
+#include "config.h"
 #include "intl.h"
+#include "util.h"
 #include "util-ank.h"
 #include "net-support.h"
 #include "version.h"
@@ -38,24 +41,22 @@ int  filter_family;
 #define NEWADDR		1
 #define DELADDR		2
 
-char *Release = RELEASE,
-     *Version = "ipmaddr 1.0",
-     *Signature = "Alexey Kuznetsov, <kuznet@ms2.inr.ac.ru>";
+static char *Release = RELEASE, *Signature = "Alexey Kuznetsov";
 
 static void version(void)
 {
-	printf("%s\n%s\n%s\n", Release, Version, Signature);
+	printf("%s\n%s\n", Release, Signature);
 	exit(E_VERSION);
 }
 
-static void usage(void) __attribute__((noreturn));
-
-static void usage(void)
+__attribute__((noreturn))
+static void usage(int rc)
 {
-	fprintf(stderr, _("Usage: ipmaddr [ add | del ] MULTIADDR dev STRING\n"));
-	fprintf(stderr, _("       ipmaddr show [ dev STRING ] [ ipv4 | ipv6 | link | all ]\n"));
-	fprintf(stderr, _("       ipmaddr -V | -version\n"));
-	exit(-1);
+	FILE *fp = rc ? stderr : stdout;
+	fprintf(fp, _("Usage: ipmaddr [ add | del ] MULTIADDR dev STRING\n"));
+	fprintf(fp, _("       ipmaddr show [ dev STRING ] [ ipv4 | ipv6 | link | all ]\n"));
+	fprintf(fp, _("       ipmaddr -V | -version\n"));
+	exit(rc);
 }
 
 static void print_lla(FILE *fp, int len, unsigned char *addr)
@@ -69,7 +70,7 @@ static void print_lla(FILE *fp, int len, unsigned char *addr)
 	}
 }
 
-static int parse_lla(char *str, unsigned char *addr)
+static int parse_lla(char *str, char *addr)
 {
 	int len=0;
 
@@ -153,8 +154,7 @@ void read_dev_mcast(struct ma_info **result_p)
 
 		len = parse_hex(hexa, (unsigned char*)&m.addr.data);
 		if (len >= 0) {
-			struct ma_info *ma = malloc(sizeof(m));
-
+			struct ma_info *ma = xmalloc(sizeof(m));
 			memcpy(ma, &m, sizeof(m));
 			ma->addr.bytelen = len;
 			ma->addr.bitlen = len<<3;
@@ -168,22 +168,21 @@ void read_dev_mcast(struct ma_info **result_p)
 
 void read_igmp(struct ma_info **result_p)
 {
-	struct ma_info m;
+	struct ma_info m, *ma = NULL;
 	char buf[256];
 	FILE *fp = fopen(_PATH_PROCNET_IGMP, "r");
 
 	if (!fp)
 		return;
 	memset(&m, 0, sizeof(m));
-	fgets(buf, sizeof(buf), fp);
+	if (fgets(buf, sizeof(buf), fp))
+		/* eat line */;
 
 	m.addr.family = AF_INET;
 	m.addr.bitlen = 32;
 	m.addr.bytelen = 4;
 
 	while (fgets(buf, sizeof(buf), fp)) {
-		struct ma_info *ma = malloc(sizeof(m));
-
 		if (buf[0] != '\t') {
 			sscanf(buf, "%d%s", &m.index, m.name);
 			continue;
@@ -194,7 +193,7 @@ void read_igmp(struct ma_info **result_p)
 
 		sscanf(buf, "%08x%d", (__u32*)&m.addr.data, &m.users);
 
-		ma = malloc(sizeof(m));
+		ma = xmalloc(sizeof(m));
 		memcpy(ma, &m, sizeof(m));
 		maddr_ins(result_p, ma);
 	}
@@ -225,8 +224,7 @@ void read_igmp6(struct ma_info **result_p)
 
 		len = parse_hex(hexa, (unsigned char*)&m.addr.data);
 		if (len >= 0) {
-			struct ma_info *ma = malloc(sizeof(m));
-
+			struct ma_info *ma = xmalloc(sizeof(m));
 			memcpy(ma, &m, sizeof(m));
 
 			ma->addr.bytelen = len;
@@ -285,13 +283,15 @@ static void print_mlist(FILE *fp, struct ma_info *list)
 static int multiaddr_list(int argc, char **argv)
 {
 	struct ma_info *list = NULL;
+	size_t l;
 
 	while (argc > 0) {
 		if (strcmp(*argv, "dev") == 0) {
 			NEXT_ARG();
-			if (filter_dev[0])
-				usage();
-			strcpy(filter_dev, *argv);
+			l = strlen(*argv);
+			if (l <= 0 || l >= sizeof(filter_dev))
+				usage(E_OPTERR);
+			strncpy(filter_dev, *argv, sizeof (filter_dev));
 		} else if (strcmp(*argv, "all") == 0) {
 			filter_family = AF_UNSPEC;
 		} else if (strcmp(*argv, "ipv4") == 0) {
@@ -301,9 +301,10 @@ static int multiaddr_list(int argc, char **argv)
 		} else if (strcmp(*argv, "link") == 0) {
 			filter_family = AF_PACKET;
 		} else {
-			if (filter_dev[0])
-				usage();
-			strcpy(filter_dev, *argv);
+			l = strlen(*argv);
+			if (l <= 0 || l >= sizeof(filter_dev))
+				usage(E_OPTERR);
+			strncpy(filter_dev, *argv, sizeof (filter_dev));
 		}
 		argv++; argc--;
 	}
@@ -334,18 +335,18 @@ int multiaddr_modify(int cmd, int argc, char **argv)
 		if (strcmp(*argv, "dev") == 0) {
 			NEXT_ARG();
 			if (ifr.ifr_name[0])
-				usage();
-			strncpy(ifr.ifr_name, *argv, IFNAMSIZ);
+				usage(E_OPTERR);
+			safe_strncpy(ifr.ifr_name, *argv, IFNAMSIZ);
 		} else {
 			if (ifr.ifr_hwaddr.sa_data[0])
-				usage();
+				usage(E_OPTERR);
 			if (parse_lla(*argv, ifr.ifr_hwaddr.sa_data) < 0)
-				usage();
+				usage(E_OPTERR);
 		}
 		argc--; argv++;
 	}
 	if (ifr.ifr_name[0] == 0)
-		usage();
+		usage(E_OPTERR);
 
 	fd = socket(AF_INET, SOCK_DGRAM, 0);
 	if (fd < 0) {
@@ -373,7 +374,7 @@ int do_multiaddr(int argc, char **argv)
 	if (matches(*argv, "list") == 0 || matches(*argv, "show") == 0
 	    || matches(*argv, "lst") == 0)
 		return multiaddr_list(argc-1, argv+1);
-	usage();
+	usage(E_OPTERR);
 }
 
 int preferred_family = AF_UNSPEC;
@@ -384,12 +385,18 @@ int main(int argc, char **argv)
 {
 	char *basename;
 
+#if I18N
+	setlocale (LC_ALL, "");
+	bindtextdomain("net-tools", "/usr/share/locale");
+	textdomain("net-tools");
+#endif
+
 	basename = strrchr(argv[0], '/');
 	if (basename == NULL)
 		basename = argv[0];
 	else
 		basename++;
-	
+
 	while (argc > 1) {
 		if (argv[1][0] != '-')
 			break;
@@ -397,13 +404,13 @@ int main(int argc, char **argv)
 			argc--;
 			argv++;
 			if (argc <= 1)
-				usage();
+				usage(E_OPTERR);
 			if (strcmp(argv[1], "inet") == 0)
 				preferred_family = AF_INET;
 			else if (strcmp(argv[1], "inet6") == 0)
 				preferred_family = AF_INET6;
 			else
-				usage();
+				usage(E_OPTERR);
 		} else if (matches(argv[1], "-stats") == 0 ||
 			   matches(argv[1], "-statistics") == 0) {
 			++show_stats;
@@ -411,8 +418,10 @@ int main(int argc, char **argv)
 			++resolve_hosts;
 		} else if ((matches(argv[1], "-V") == 0) || matches(argv[1], "--version") == 0) {
 			version();
+		} else if ((matches(argv[1], "-h") == 0) || matches(argv[1], "--help") == 0) {
+			usage(E_USAGE);
 		} else
-			usage();
+			usage(E_OPTERR);
 		argc--;	argv++;
 	}
 
