@@ -2,13 +2,13 @@
  * lib/ppp.c  This file contains the SLIP support for the NET-2 base
  *              distribution.
  *
- * Version:     $Id: ppp.c,v 1.3 1998/11/15 20:11:45 freitag Exp $
+ * Version:     $Id: ppp.c,v 1.4 2000/03/05 11:26:03 philip Exp $
  *
  * Author:      Fred N. van Kempen, <waltje@uwalt.nl.mugnet.org>
  *              Copyright 1993 MicroWalt Corporation
  *
  *              Modified by Alan Cox, May 94 to cover NET-3
- *                       
+ *
  * Changes:
  * 980701 {1.12} Arnaldo Carvalho de Melo - GNU gettext instead of catgets
  *
@@ -46,12 +46,10 @@ static int do_ppp(int fd)
 }
 
 
-
-
 struct hwtype ppp_hwtype =
 {
     "ppp", NULL, /*"Point-Point Protocol", */ ARPHRD_PPP, 0,
-    NULL, NULL, NULL, do_ppp
+    NULL, NULL, do_ppp, 0
 };
 
 
